@@ -81,13 +81,13 @@ DEFINE_TEST(open_close_session)
 	GAsyncResult *result = NULL;
 	GError *err = NULL;
 	
-	sess = gp11_slot_open_session_full (slot, 0, NULL, &err);
+	sess = gp11_slot_open_session_full (slot, 0, NULL, NULL, NULL, &err);
 	SUCCESS_RES (sess, err);
 	
 	g_object_unref (sess);
 	
 	/* Test opening async */
-	gp11_slot_open_session_async (slot, 0, NULL, fetch_async_result, &result);
+	gp11_slot_open_session_async (slot, 0, NULL, NULL, NULL, fetch_async_result, &result);
 	
 	WAIT_UNTIL (result);
 	g_assert (result != NULL);
@@ -130,7 +130,7 @@ DEFINE_TEST(open_reused)
 	g_object_unref (sess);
 	
 	/* Test opening async */
-	gp11_slot_open_session_async (slot, 0, NULL, fetch_async_result, &result);
+	gp11_slot_open_session_async (slot, 0, NULL, NULL, NULL, fetch_async_result, &result);
 	WAIT_UNTIL (result);
 	g_assert (result != NULL);
 	sess = gp11_slot_open_session_finish (slot, result, &err);
@@ -160,6 +160,40 @@ DEFINE_TEST(open_reused)
 }
 
 
+DEFINE_TEST(init_set_pin)
+{
+	GAsyncResult *result = NULL;
+	GError *err = NULL;
+	gboolean ret;
+
+	/* init pin */
+	ret = gp11_session_init_pin (session, (guchar*)"booo", 4, &err);
+	SUCCESS_RES (ret, err);
+
+	/* set pin */
+	ret = gp11_session_set_pin (session, (guchar*)"booo", 4, (guchar*)"tooo", 4, &err);
+	SUCCESS_RES (ret, err);
+
+	/* init pin async */
+	gp11_session_init_pin_async (session, (guchar*)"booo", 4, NULL, fetch_async_result, &result);
+	WAIT_UNTIL (result);
+	g_assert (result != NULL);
+	ret = gp11_session_init_pin_finish (session, result, &err);
+	SUCCESS_RES (ret, err);
+	g_object_unref (result);
+	result = NULL;
+
+	/* set pin async */
+	gp11_session_set_pin_async (session, (guchar*)"booo", 4, (guchar*)"tooo", 4, NULL, fetch_async_result, &result);
+	WAIT_UNTIL (result);
+	g_assert (result != NULL);
+	ret = gp11_session_set_pin_finish (session, result, &err);
+	SUCCESS_RES (ret, err);
+	g_object_unref (result);
+	result = NULL;
+}
+
+
 DEFINE_TEST(login_logout)
 {
 	GAsyncResult *result = NULL;
@@ -262,7 +296,7 @@ DEFINE_TEST(auto_login)
 	SUCCESS_RES (ret, err);
 	
 	/* Now try the same thing, but asyncronously */
-	gp11_slot_open_session_async (slot, CKF_RW_SESSION, NULL, fetch_async_result, &result);
+	gp11_slot_open_session_async (slot, CKF_RW_SESSION, NULL, NULL, NULL, fetch_async_result, &result);
 	WAIT_UNTIL (result);
 	g_assert (result != NULL);
 	new_session = gp11_slot_open_session_finish (slot, result, &err);
